//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Core/Contrib/ParticleLayoutContribution.cpp
//! @brief     Implements class ParticleLayoutContribution.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Core/Contrib/ParticleLayoutContribution.h"
#include "Resample/Element/DiffuseElement.h"
#include "Resample/Interparticle/DecouplingApproximationStrategy.h"
#include "Resample/Interparticle/SSCAStrategy.h"
#include "Resample/Processed/ProcessedLayout.h"
#include "Sample/Aggregate/InterferenceFunctionRadialParaCrystal.h"

namespace {

std::unique_ptr<IInterparticleStrategy>
processedInterferenceFunction(const ProcessedLayout& re_layout, const SimulationOptions& sim_params,
                              bool polarized)
{
    const IInterferenceFunction* iff = re_layout.interferenceFunction();
    if (iff && re_layout.numberOfSlices() > 1 && !iff->supportsMultilayer())
        throw std::runtime_error("LayoutStrategyBuilder::checkInterferenceFunction: "
                                 "interference function does not support multiple layers");

    const std::vector<std::unique_ptr<const CoherentFFSum>>& weighted_formfactors =
        re_layout.formFactorList();

    auto radial_para = dynamic_cast<const InterferenceFunctionRadialParaCrystal*>(iff);
    if (radial_para && radial_para->kappa() > 0.0) {
        double kappa = radial_para->kappa();
        return std::make_unique<SSCAStrategy>(weighted_formfactors, radial_para, sim_params,
                                              polarized, kappa);
    }

    return std::make_unique<DecouplingApproximationStrategy>(weighted_formfactors, iff, sim_params,
                                                             polarized);
}

} // namespace


ParticleLayoutContribution::ParticleLayoutContribution(const ProcessedLayout& layout,
                                                       const SimulationOptions& options,
                                                       bool polarized)
    : m_layout(layout)
    , m_interparticle_strategy(processedInterferenceFunction(layout, options, polarized))
{
}

ParticleLayoutContribution::~ParticleLayoutContribution() = default;

void ParticleLayoutContribution::compute(DiffuseElement& ele) const
{
    ele.addIntensity(m_interparticle_strategy->evaluate(ele) * m_layout.surfaceDensity());
}
