//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/FormFactorItems.h
//! @brief     Defines FormFactorItems classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_FORMFACTORITEMS_H
#define BORNAGAIN_GUI_MODELS_FORMFACTORITEMS_H

#include "GUI/Models/SessionItem.h"

class IFormFactor;

class BA_CORE_API_ FormFactorItem : public SessionItem {
protected:
    explicit FormFactorItem(const QString& model_type) : SessionItem(model_type) {}

public:
    virtual std::unique_ptr<IFormFactor> createFormFactor() const = 0;
};

class BA_CORE_API_ AnisoPyramidItem : public FormFactorItem {
private:
    static const QString P_LENGTH;
    static const QString P_WIDTH;
    static const QString P_HEIGHT;
    static const QString P_ALPHA;

public:
    static const QString M_TYPE;

    AnisoPyramidItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setLength(double length);
    void setWidth(double width);
    void setHeight(double height);
    void setAlpha(double alpha);
};

class BA_CORE_API_ BarGaussItem : public FormFactorItem {
private:
    static const QString P_LENGTH;
    static const QString P_WIDTH;
    static const QString P_HEIGHT;

public:
    static const QString M_TYPE;

    BarGaussItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setLength(double length);
    void setWidth(double width);
    void setHeight(double height);
};

class BA_CORE_API_ BarLorentzItem : public FormFactorItem {
private:
    static const QString P_LENGTH;
    static const QString P_WIDTH;
    static const QString P_HEIGHT;

public:
    static const QString M_TYPE;

    BarLorentzItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setLength(double length);
    void setWidth(double width);
    void setHeight(double height);
};

class BA_CORE_API_ BoxItem : public FormFactorItem {
private:
    static const QString P_LENGTH;
    static const QString P_WIDTH;
    static const QString P_HEIGHT;

public:
    static const QString M_TYPE;

    BoxItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setLength(double length);
    void setWidth(double width);
    void setHeight(double height);
};

class BA_CORE_API_ ConeItem : public FormFactorItem {
private:
    static const QString P_RADIUS;
    static const QString P_HEIGHT;
    static const QString P_ALPHA;

public:
    static const QString M_TYPE;

    ConeItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setRadius(double radius);
    void setHeight(double height);
    void setAlpha(double alpha);
};

class BA_CORE_API_ Cone6Item : public FormFactorItem {
private:
    static const QString P_BASEEDGE;
    static const QString P_HEIGHT;
    static const QString P_ALPHA;

public:
    static const QString M_TYPE;

    Cone6Item();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setBaseEdge(double base_edge);
    void setHeight(double height);
    void setAlpha(double alpha);
};

class BA_CORE_API_ CuboctahedronItem : public FormFactorItem {
private:
    static const QString P_LENGTH;
    static const QString P_HEIGHT;
    static const QString P_HEIGHT_RATIO;
    static const QString P_ALPHA;

public:
    static const QString M_TYPE;

    CuboctahedronItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setLength(double length);
    void setHeight(double height);
    void setHeightRatio(double height_ratio);
    void setAlpha(double alpha);
};

class BA_CORE_API_ CylinderItem : public FormFactorItem {
private:
    static const QString P_RADIUS;
    static const QString P_HEIGHT;

public:
    static const QString M_TYPE;

    CylinderItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setRadius(double radius);
    void setHeight(double height);

    SessionItem* radiusItem() const;
    SessionItem* heightItem() const;
};

class BA_CORE_API_ DodecahedronItem : public FormFactorItem {
private:
    static const QString P_EDGE;

public:
    static const QString M_TYPE;

    DodecahedronItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setEdge(double edge);
};

class BA_CORE_API_ EllipsoidalCylinderItem : public FormFactorItem {
private:
    static const QString P_RADIUS_X;
    static const QString P_RADIUS_Y;
    static const QString P_HEIGHT;

public:
    static const QString M_TYPE;

    EllipsoidalCylinderItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setRadiusX(double radius_x);
    void setRadiusY(double radius_y);
    void setHeight(double height);
};

class BA_CORE_API_ FullSphereItem : public FormFactorItem {
private:
    static const QString P_RADIUS;

public:
    static const QString M_TYPE;

    FullSphereItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    double radius() const;
    void setRadius(double radius);
    SessionItem* radiusItem() const;
};

class BA_CORE_API_ FullSpheroidItem : public FormFactorItem {
private:
    static const QString P_RADIUS;
    static const QString P_HEIGHT;

public:
    static const QString M_TYPE;

    FullSpheroidItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setRadius(double radius);
    void setHeight(double height);
};

class BA_CORE_API_ HemiEllipsoidItem : public FormFactorItem {
private:
    static const QString P_RADIUS_X;
    static const QString P_RADIUS_Y;
    static const QString P_HEIGHT;

public:
    static const QString M_TYPE;

    HemiEllipsoidItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setRadiusX(double radius_x);
    void setRadiusY(double radius_y);
    void setHeight(double height);
};

class BA_CORE_API_ IcosahedronItem : public FormFactorItem {
private:
    static const QString P_EDGE;

public:
    static const QString M_TYPE;

    IcosahedronItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setEdge(double edge);
};

class BA_CORE_API_ Prism3Item : public FormFactorItem {
private:
    static const QString P_BASEEDGE;
    static const QString P_HEIGHT;

public:
    static const QString M_TYPE;

    Prism3Item();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setBaseEdge(double base_edge);
    void setHeight(double height);
};

class BA_CORE_API_ Prism6Item : public FormFactorItem {
private:
    static const QString P_BASEEDGE;
    static const QString P_HEIGHT;

public:
    static const QString M_TYPE;

    Prism6Item();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setBaseEdge(double base_edge);
    void setHeight(double height);
};

class BA_CORE_API_ PyramidItem : public FormFactorItem {
private:
    static const QString P_BASEEDGE;
    static const QString P_HEIGHT;
    static const QString P_ALPHA;

public:
    static const QString M_TYPE;

    PyramidItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setBaseEdge(double base_edge);
    void setHeight(double height);
    void setAlpha(double alpha);
};

class BA_CORE_API_ CosineRippleBoxItem : public FormFactorItem {
private:
    static const QString P_LENGTH;
    static const QString P_WIDTH;
    static const QString P_HEIGHT;

public:
    static const QString M_TYPE;

    CosineRippleBoxItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setLength(double length);
    void setWidth(double width);
    void setHeight(double height);
};

class BA_CORE_API_ CosineRippleGaussItem : public FormFactorItem {
private:
    static const QString P_LENGTH;
    static const QString P_WIDTH;
    static const QString P_HEIGHT;

public:
    static const QString M_TYPE;

    CosineRippleGaussItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setLength(double length);
    void setWidth(double width);
    void setHeight(double height);
};

class BA_CORE_API_ CosineRippleLorentzItem : public FormFactorItem {
private:
    static const QString P_LENGTH;
    static const QString P_WIDTH;
    static const QString P_HEIGHT;

public:
    static const QString M_TYPE;

    CosineRippleLorentzItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setLength(double length);
    void setWidth(double width);
    void setHeight(double height);
};

class BA_CORE_API_ SawtoothRippleBoxItem : public FormFactorItem {
private:
    static const QString P_LENGTH;
    static const QString P_WIDTH;
    static const QString P_HEIGHT;
    static const QString P_ASYMMETRY;

public:
    static const QString M_TYPE;

    SawtoothRippleBoxItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setLength(double length);
    void setWidth(double width);
    void setHeight(double height);
    void setAsymmetry(double asymmetry);
};

class BA_CORE_API_ SawtoothRippleGaussItem : public FormFactorItem {
private:
    static const QString P_LENGTH;
    static const QString P_WIDTH;
    static const QString P_HEIGHT;
    static const QString P_ASYMMETRY;

public:
    static const QString M_TYPE;

    SawtoothRippleGaussItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setLength(double length);
    void setWidth(double width);
    void setHeight(double height);
    void setAsymmetry(double asymmetry);
};

class BA_CORE_API_ SawtoothRippleLorentzItem : public FormFactorItem {
private:
    static const QString P_LENGTH;
    static const QString P_WIDTH;
    static const QString P_HEIGHT;
    static const QString P_ASYMMETRY;

public:
    static const QString M_TYPE;

    SawtoothRippleLorentzItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setLength(double length);
    void setWidth(double width);
    void setHeight(double height);
    void setAsymmetry(double asymmetry);
};

class BA_CORE_API_ TetrahedronItem : public FormFactorItem {
private:
    static const QString P_BASEEDGE;
    static const QString P_HEIGHT;
    static const QString P_ALPHA;

public:
    static const QString M_TYPE;

    TetrahedronItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setBaseEdge(double base_edge);
    void setHeight(double height);
    void setAlpha(double alpha);
};

class BA_CORE_API_ TruncatedCubeItem : public FormFactorItem {
private:
    static const QString P_LENGTH;
    static const QString P_REMOVED_LENGTH;

public:
    static const QString M_TYPE;

    TruncatedCubeItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setLength(double length);
    void setRemovedLength(double removed_length);
};

class BA_CORE_API_ TruncatedSphereItem : public FormFactorItem {
private:
    static const QString P_RADIUS;
    static const QString P_HEIGHT;
    static const QString P_REMOVED_TOP;

public:
    static const QString M_TYPE;

    TruncatedSphereItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setRadius(double radius);
    void setHeight(double height);
    void setRemovedTop(double removed_top);
};

class BA_CORE_API_ TruncatedSpheroidItem : public FormFactorItem {
private:
    static const QString P_RADIUS;
    static const QString P_HEIGHT;
    static const QString P_HFC;
    static const QString P_REMOVED_TOP;

public:
    static const QString M_TYPE;

    TruncatedSpheroidItem();
    std::unique_ptr<IFormFactor> createFormFactor() const override;

    void setRadius(double radius);
    void setHeight(double height);
    void setHeightFlattening(double height_flattening);
    void setRemovedTop(double removed_top);
};

#endif // BORNAGAIN_GUI_MODELS_FORMFACTORITEMS_H
