//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Resample/Specular/SpecularScalarStrategy.cpp
//! @brief     Implements class SpecularScalarStrategy.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Resample/Specular/SpecularScalarStrategy.h"
#include "Base/Math/Constants.h"
#include "Base/Math/Functions.h"
#include "Base/Utils/Assert.h"
#include "Resample/Flux/ScalarFlux.h"
#include "Resample/Slice/KzComputation.h"
#include "Resample/Slice/Slice.h"
#include "Resample/Slice/SliceStack.h"
#include "Sample/Interface/LayerRoughness.h"
#include "Sample/Multilayer/Layer.h"
#include "Sample/Multilayer/RoughnessModels.h"
#include <Eigen/Dense>
#include <stdexcept>

namespace {

std::pair<complex_t, complex_t> transition(complex_t kzi, complex_t kzi1, double sigma,
                                           const RoughnessModel& r_model)
{
    if (r_model == RoughnessModel::NEVOT_CROCE) {
        // Roughness is modelled by a Gaussian profile, i.e. Nevot-Croce factors for the
        // reflection coefficients.
        // Implementation follows A. Gibaud and G. Vignaud, in X-ray and Neutron Reflectivity,
        // edited by J. Daillant and A. Gibaud, volume 770 of Lecture Notes in Physics (2009)
        complex_t roughness_diff = 1;
        complex_t roughness_sum = 1;
        if (sigma > 0.0) {
            roughness_diff = std::exp(-(kzi1 - kzi) * (kzi1 - kzi) * sigma * sigma / 2.);
            roughness_sum = std::exp(-(kzi1 + kzi) * (kzi1 + kzi) * sigma * sigma / 2.);
        }
        const complex_t kz_ratio = kzi1 / kzi;

        const complex_t a00 = 0.5 * (1. + kz_ratio) * roughness_diff;
        const complex_t a01 = 0.5 * (1. - kz_ratio) * roughness_sum;

        return {a00, a01};

    } else { // TANH model assumed
        // Roughness is modelled by tanh profile
        // [e.g. Bahr, Press, et al, Phys. Rev. B, vol. 47 (8), p. 4385 (1993)].
        complex_t roughness = 1;
        if (sigma > 0.0) {
            const double sigeff = std::pow(M_PI_2, 1.5) * sigma;
            roughness = std::sqrt(Math::tanhc(sigeff * kzi1) / Math::tanhc(sigeff * kzi));
        }
        const complex_t inv_roughness = 1.0 / roughness;
        const complex_t kz_ratio = kzi1 / kzi * roughness;

        const complex_t a00 = 0.5 * (inv_roughness + kz_ratio);
        const complex_t a01 = 0.5 * (inv_roughness - kz_ratio);

        return {a00, a01};
    }
}

std::vector<Eigen::Vector2cd> computeTR(const SliceStack& slices,
                                        const std::vector<complex_t>& kz,
                                        const RoughnessModel& r_model)
{
    const size_t N = slices.size();
    std::vector<Eigen::Vector2cd> TR(N, {1., 0.});

    if (N == 1) // If only one layer present, there's nothing left to calculate
        return TR;

    if (kz[0] == 0.0) { // If kz in layer 0 is zero, R0 = -T0 and all others equal to 0
        TR[0] = {1.0, -1.0};
        for (size_t i = 1; i < N; ++i)
            TR[i].setZero();
        return TR;
    }

    // Calculate transmission/refraction coefficients t_r for each layer, from bottom to top.
    TR.back() = {1.0, 0.0};
    std::vector<complex_t> factors(N - 1);
    for (int i = N - 2; i >= 0; i--) {
        double sigma = 0.0;
        if (const auto roughness = slices.bottomRoughness(i))
            sigma = roughness->getSigma();

        const auto [mp, mm] = ::transition(kz[i], kz[i + 1], sigma, r_model);

        const complex_t delta = exp_I(kz[i] * slices[i].thicknessOr0());

        complex_t S = mp + mm * TR[i + 1](1);
        S = 1. / S * delta;
        factors[i] = S;

        TR[i](1) = delta * (mm + mp * TR[i + 1](1)) * S;
    }

    // Now correct all amplitudes by dividing by the remaining factors in forward direction.
    // At some point this divison underflows; from there on all further amplitudes are set to zero.
    complex_t dampingFactor = 1;
    for (size_t j = 1; j < N; ++j) {
        dampingFactor = dampingFactor * factors[j - 1];

        TR[j](0) = dampingFactor;
        TR[j](1) *= dampingFactor;
    }

    return TR;
}

} // namespace

//  ************************************************************************************************
//  namespace implementation
//  ************************************************************************************************

Fluxes Compute::SpecularScalar::Execute(const SliceStack& slices, const kvector_t& k)
{
    std::vector<complex_t> kz = Compute::Kz::computeReducedKz(slices, k);
    ASSERT(slices.size() == kz.size());

    const std::vector<Eigen::Vector2cd> TR = computeTR(slices, kz, slices.roughnessModel());

    Fluxes result;
    for (size_t i = 0; i < kz.size(); ++i)
        result.emplace_back(std::make_unique<ScalarFlux>(kz[i], TR[i]));
    return result;
}

complex_t Compute::SpecularScalar::computeTopLayerR(
    const SliceStack& slices, const std::vector<complex_t>& kz,
    const RoughnessModel& r_model)
{
    ASSERT(slices.size() == kz.size());

    const size_t N = slices.size();

    if (N == 1)
        return 0.; // only one layer present, there's nothing left to calculate
    if (kz[0] == 0.)
        return -1.;

    complex_t R_i1 = 0.;

    for (int i = N - 2; i >= 0; i--) {
        double sigma = 0.0;
        if (const auto roughness = slices.bottomRoughness(i))
            sigma = roughness->getSigma();

        const auto [mp, mm] = ::transition(kz[i], kz[i + 1], sigma, r_model);

        const complex_t delta = exp_I(kz[i] * slices[i].thicknessOr0());

        complex_t S = mp + mm * R_i1;
        S = 1. / S * delta;

        R_i1 = delta * (mm + mp * R_i1) * S;
    }

    return R_i1;
}
